# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet_x/enterprisemodules/weblogic/functions'

newproperty(:mapper_specific) do
  include EasyType
  include Puppet_X::EnterpriseModules::WebLogic::Functions

  desc <<-DESC
    The mapper specific properties.

    This value is a Hash that contains key's and values. Here is an example on how to use this:


        wls_credential_mapper { 'dom0/MyMapper':
          ensure => present,
          mapperclassname => 'weblogic.security.providers.credentials.PKICredentialMapper',
          mapper_specific => {
            'KeyStoreType' => 'JKS',
            'KeyStoreFileName' => '/a.aaaa',
            'KeyStorePassPhrase' => 'secretstuff',
            'UseResourceHierarchy' => 1,
            'UseInitiatorGroupNames' => 0,
          },
        }



      In this example the `KeyStoreType`, `KeyStoreFileName`, `KeyStorePassPhrase` `UseInitiatorGroupNames` and
      `UseResourceHierarchy` are the mapper-specific properties for the `weblogic.security.providers.credentials.PKICredentialMapper`
      mapper. In this example, we mix integer values and quoted integer values. It doesn't matter. Internally
      all values are quoted and converted to their right representation.

      For the logical value `UseResourceHierarchy` and `UseInitiatorGroupNames`, you must use `0` for false and `1` for true.#{' '}
      When you use the values  `false` or `true`, the changes are not idempotent.

      When you specify a key value that the provider doesn't understand or a value that is invalid for the
      specific property, the puppet type will fail with the error

      `Error: Failed to apply catalog: command in daemon failed.`


  DESC

  data_type('Optional[String]')

  defaultto({})
  # rubocop:disable Naming/MethodParameterName
  def insync?(is)
    is.to_a.sort == should.to_a.sort
  end
  # rubocop:enable Naming/MethodParameterName
  validate do |value|
    fail 'should be a Hash value' unless value.is_a?(Hash)
  end

  #
  # Make sure the key and the values in the hashes are strings
  #
  munge do |value_hash|
    value_hash.each do |key, value|
      value_hash[key.to_s] = value.to_s
    end
    value_hash
  end

  def change_to_s(from, to)
    return_value = []
    realm_path = wls_get_realm_path('', provider)
    base_path = "#{realm_path}/CredentialMappers/#{resource['credential_mapper_name']}"
    from.each_key do |property|
      next unless to[property] != from[property]

      full_key = "#{base_path}/#{property}"
      #
      # If the key is encrypted, we don't show it's content
      #
      return_value << if wls_is_encrypted(full_key, provider) == '1'
                        "property #{property} from [old value redacted] to [new value redacted]"
                      else
                        "property #{property} from #{from[property]} to #{to[property]}"
                      end
    end
    "Changed #{return_value.join(' and ')}."
  end
end
